package nl.sbdeveloper.themeparkplus;

import club.minnced.discord.webhook.WebhookClient;
import club.minnced.discord.webhook.WebhookClientBuilder;
import net.milkbowl.vault.economy.Economy;
import nl.SBDevelopment.SBUtilities.Data.YamlFile;
import nl.SBDevelopment.SBUtilities.Logger.Logger;
import nl.SBDevelopment.SBUtilities.PrivateManagers.UpdateManager;
import nl.SBDevelopment.SBUtilities.SBUtilities;
import nl.sbdeveloper.themeparkplus.commands.TPPCMD;
import nl.sbdeveloper.themeparkplus.listeners.AntiFreerunListener;
import nl.sbdeveloper.themeparkplus.listeners.DirectionalGateListener;
import nl.sbdeveloper.themeparkplus.listeners.FastpassListeners;
import nl.sbdeveloper.themeparkplus.listeners.StatusChangeListener;
import nl.sbdeveloper.themeparkplus.managers.DBManager;
import nl.sbdeveloper.themeparkplus.util.LGUtil;
import nl.sbdeveloper.themeparkplus.util.License;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.RegisteredServiceProvider;
import org.bukkit.plugin.java.JavaPlugin;
import org.jetbrains.annotations.NotNull;

import java.io.InputStreamReader;
import java.io.Reader;
import java.lang.reflect.InvocationTargetException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Objects;

public final class ThemeParkPlus extends JavaPlugin {

    private static ThemeParkPlus instance;
    private static YamlFile config;
    private static DBManager data;
    private static YamlFile messages;

    private static Economy econ = null;
    private static WebhookClient webhookClient;

    private int configVersion = 1;

    @Override
    public void onEnable() {
        instance = this;

        new SBUtilities(this, "ThemeParkPlus");

        Logger.logInfo("-------------------------------", true);
        Logger.logInfo("ThemeParkPlus v" + this.getDescription().getVersion(), true);
        Logger.logInfo("Made by SBDeveloper", true);

        Logger.logInfo(" ", true);

        Logger.logInfo("Loading Files...", true);

        config = new YamlFile("config");
        config.loadDefaults();

        data = new DBManager("data");

        messages = new YamlFile("messages");
        messages.loadDefaults();

        Reader defConfigStream = new InputStreamReader(Objects.requireNonNull(getResource("config.yml")), StandardCharsets.UTF_8);
        YamlConfiguration defConfig = YamlConfiguration.loadConfiguration(defConfigStream);
        config.getFile().setDefaults(defConfig);
        config.getFile().options().copyDefaults(true);
        config.saveFile();

        if (!config.getFile().contains("Version") || Objects.equals(config.getFile().getString("Version"), "")) {
            config.getFile().set("Version", configVersion);
            config.saveFile();
        }

        if (!Objects.equals(config.getFile().getString("Version"), String.valueOf(configVersion))) {
            Logger.logInfo("Updating outdated config...", true);
            updateConfig();
        }

        Logger.logInfo("Checking license...", true);

        if (config.getFile().contains("License")) {
            Logger.logInfo("Licence code: " + config.getFile().getString("License"), true);
        } else {
            Logger.logError("Licence code unknown! Please change the config.yml!", true);
            return;
        }

        new License(this, "TP", config.getFile().getString("License"));

        new UpdateManager(this, 6, UpdateManager.CheckType.SBDPLUGINS).handleResponse((versionResponse, version) -> {
            if (versionResponse == UpdateManager.VersionResponse.FOUND_NEW) {
                Logger.logWarning("There is a new version available! Curent: " + this.getDescription().getVersion() + " New: " + version, true);
            } else if (versionResponse == UpdateManager.VersionResponse.LATEST) {
                Logger.logInfo("You are running the latest version [" + this.getDescription().getVersion() + "]!", true);
            } else if (versionResponse == UpdateManager.VersionResponse.UNAVAILABLE) {
                Logger.logError("Unable to perform an update check.", true);
            }
        }).check();

        if (Bukkit.getPluginManager().getPlugin("ThemePark") == null) {
            Logger.logError("Missing ThemePark! Please install it first.", true);
            getServer().getPluginManager().disablePlugin(this);
            return;
        }

        if (Bukkit.getPluginManager().getPlugin("WorldEdit") == null) {
            Logger.logError("Missing WorldEdit! Please install it first.", true);
            getServer().getPluginManager().disablePlugin(this);
            return;
        }

        if (!setupEconomy()) {
            Logger.logError("Missing Vault! Please install it first.", true);
            getServer().getPluginManager().disablePlugin(this);
            return;
        }

        Logger.logInfo("Loading commands...", true);
        Objects.requireNonNull(getCommand("themeparkplus"), "Couldn't read command from plugin.yml!").setExecutor(new TPPCMD());

        Logger.logInfo("Loading listeners...", true);
        Bukkit.getPluginManager().registerEvents(new DirectionalGateListener(), this);
        Bukkit.getPluginManager().registerEvents(new FastpassListeners(), this);
        if (getSConfig().getFile().getBoolean("AntiFreerun.Enabled")) {
            Bukkit.getPluginManager().registerEvents(new AntiFreerunListener(), this);
        }

        if (getSConfig().getFile().getBoolean("DiscordWebhook.Enabled")) {
            String URL = getSConfig().getFile().getString("DiscordWebhook.WebhookURL");
            if (URL != null) {
                Bukkit.getPluginManager().registerEvents(new StatusChangeListener(), this);

                Logger.logInfo("Loading Discord webhook...", true);
                WebhookClientBuilder builder = new WebhookClientBuilder(URL);
                builder.setThreadFactory((job) -> {
                    Thread thread = new Thread(job);
                    thread.setName("Hello");
                    thread.setDaemon(true);
                    return thread;
                });
                builder.setWait(true);
                webhookClient = builder.build();
            } else {
                Logger.logError("Couldn't load the webhook builder! The URL is null.", true);
            }
        }

        Logger.logInfo("Loading Lamp & Gate utils...", true);
        try {
            new LGUtil();
        } catch (ClassNotFoundException | NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
            e.printStackTrace();
            Logger.logError("Couldn't find classes for Lamp & Gate util. The plugin won't work as intended.", true);
        }

        Logger.logInfo("Plugin enabled!", true);
        Logger.logInfo("-------------------------------", true);
    }

    @Override
    public void onDisable() {
        if (getSConfig().getFile().getBoolean("DiscordWebhook.Enabled")) {
            webhookClient.close();
        }
        Logger.logInfo("Plugin disabled!", true);
        instance = null;
    }

    public static ThemeParkPlus getInstance() {
        return instance;
    }

    public static YamlFile getSConfig() {
        return config;
    }

    public static DBManager getData() {
        return data;
    }

    public static YamlFile getMessages() {
        return messages;
    }

    public static Economy getEconomy() {
        return econ;
    }

    public static WebhookClient getWebhookClient() {
        return webhookClient;
    }

    private boolean setupEconomy() {
        if (getServer().getPluginManager().getPlugin("Vault") == null) {
            return false;
        }
        RegisteredServiceProvider<Economy> rsp = getServer().getServicesManager().getRegistration(Economy.class);
        if (rsp == null) {
            return false;
        }
        econ = rsp.getProvider();
        return econ != null;
    }

    private void updateConfig() {
        HashMap< String, Object > newConfig = getConfigVals();
        for (String var: config.getFile().getKeys(false)) {
            newConfig.remove(var);
        }
        if (newConfig.size() != 0) {
            for (String key: newConfig.keySet()) {
                config.getFile().set(key, newConfig.get(key));
            }
            config.getFile().set("Version", configVersion);
            config.saveFile();
        }
    }

    @NotNull
    private HashMap<String, Object> getConfigVals() {
        HashMap<String, Object> var = new HashMap <>();
        Reader defConfigStream = new InputStreamReader(Objects.requireNonNull(getResource("config.yml")), StandardCharsets.UTF_8);
        YamlConfiguration defConfig = YamlConfiguration.loadConfiguration(defConfigStream);
        for (String key : defConfig.getKeys(false)) {
            var.put(key, defConfig.get(key));
        }
        return var;
    }
}
