package nl.sbdeveloper.themeparkplus.api.objects;

import lombok.Getter;
import lombok.Setter;
import nl.sbdeveloper.themeparkplus.api.enums.WalkingDirection;
import org.bukkit.Location;

@Getter @Setter
public class Gate {
    private final Location loc;

    /* If max count */
    private boolean hasMaxCount = false;
    private int passedCount = 0;
    private int maxCount;

    /* If directional */
    private boolean isDirectional = false;
    private WalkingDirection direction;

    /**
     * Constructor for a normal gate.
     *
     * @param loc The gate location
     */
    public Gate(Location loc) {
        this.loc = loc;
    }

    /**
     * Constructor for a directional gate.
     *
     * @param loc The gate location
     * @param direction The walking direction
     */
    public Gate(Location loc, WalkingDirection direction) {
        this.loc = loc;
        this.isDirectional = true;
        this.direction = direction;
    }

    /**
     * Constructor for a max count gate.
     *
     * @param loc The gate location
     * @param maxCount The max player count
     */
    public Gate(Location loc, int maxCount) {
        this.loc = loc;
        this.hasMaxCount = true;
        this.maxCount = maxCount;
    }

    /**
     * Constructor for a directional max count gate.
     *
     * @param loc The gate location
     * @param maxCount The max player count
     * @param direction The walking direction
     */
    public Gate(Location loc, int maxCount, WalkingDirection direction) {
        this.loc = loc;
        this.hasMaxCount = true;
        this.isDirectional = true;
        this.maxCount = maxCount;
        this.direction = direction;
    }
}
