package nl.sbdeveloper.themeparkplus.listeners;

import de.tr7zw.changeme.nbtapi.NBTItem;
import me.paradoxpixel.themepark.api.API;
import me.paradoxpixel.themepark.api.attraction.Attraction;
import me.paradoxpixel.themepark.api.attraction.component.Status;
import nl.sbdeveloper.themeparkplus.ThemeParkPlus;
import nl.sbdeveloper.themeparkplus.util.ConfigUtil;
import nl.sbdeveloper.themeparkplus.util.LGUtil;
import nl.sbdeveloper.themeparkplus.util.XMaterial;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.block.SignChangeEvent;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.inventory.EquipmentSlot;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.Collections;

public class FastpassListeners implements Listener {
    @EventHandler
    public void onCreate(SignChangeEvent e) {
        Player p = e.getPlayer();
        String[] lines = e.getLines();

        if (!p.hasPermission("tpp.fastpass.create")) {
            p.sendMessage(ChatColor.RED + "You don't have the permission to do this.");
            return;
        }

        if (!API.isAttraction(e.getLine(2))) {
            p.sendMessage(ChatColor.RED + e.getLine(2) + " is not an existing ride.");
            return;
        }

        if (lines[0].equalsIgnoreCase("[ThemeParkPlus]") && lines[1].equalsIgnoreCase("Machine") && !lines[2].isEmpty() && !lines[3].isEmpty()) {
            e.setLine(0, ChatColor.GRAY + "[" + ChatColor.GOLD + "ThemeParkPlus" + ChatColor.GRAY + "]");
            e.setLine(1, ChatColor.BLUE + "Machine");
        } else if (lines[0].equalsIgnoreCase("[ThemeParkPlus]") && lines[1].equalsIgnoreCase("Scanner") && !lines[2].isEmpty() && !lines[3].isEmpty()) {
            e.setLine(0, ChatColor.GRAY + "[" + ChatColor.GOLD + "ThemeParkPlus" + ChatColor.GRAY + "]");
            e.setLine(1, ChatColor.RED + "Scanner");
        }
    }

    @EventHandler
    public void onSignClick(PlayerInteractEvent e) {
        if (e.getClickedBlock() == null || e.getAction() != Action.RIGHT_CLICK_BLOCK || !(e.getClickedBlock().getState() instanceof Sign) || e.getHand() != EquipmentSlot.HAND) return;

        Sign sign = (Sign) e.getClickedBlock().getState();
        if (ChatColor.stripColor(sign.getLine(1)).equalsIgnoreCase("Machine")) {
            //Buy a ticket
            String attID = sign.getLine(2);
            if (!API.isAttraction(attID)) return;

            Attraction att = API.getAttraction(attID);

            double price;
            try {
                price = Double.parseDouble(sign.getLine(3));
            } catch (NumberFormatException ex) {
                return;
            }

            ItemStack ticket = XMaterial.PAPER.parseItem();
            if (ticket == null) return;
            ItemMeta meta = ticket.getItemMeta();
            if (meta == null) return;
            meta.setDisplayName(ChatColor.GOLD + "Fasspass Ticket");
            meta.setLore(Collections.singletonList(ChatColor.GOLD + "For: " + ConfigUtil.makecolored(att.getName())));
            ticket.setItemMeta(meta);

            NBTItem item = new NBTItem(ticket);
            item.setString("RideID", attID);
            item.setDouble("Price", price);
            ticket = item.getItem();

            if (ThemeParkPlus.getEconomy().getBalance(e.getPlayer()) < price) {
                e.getPlayer().sendMessage(ChatColor.RED + "You can't pay this fastpass ticket.");
                return;
            }

            for (ItemStack content : e.getPlayer().getInventory().getContents()) {
                if (content == null || content.getType() == null || content.getType() == Material.AIR) continue;
                NBTItem nbtContent = new NBTItem(content);
                if (content.getType() == XMaterial.PAPER.parseMaterial() && content.hasItemMeta() && nbtContent.hasKey("RideID") && nbtContent.getString("RideID").equalsIgnoreCase(attID)) {
                    e.getPlayer().sendMessage(ChatColor.RED + "You already have a ticket for this ride.");
                    return;
                }
            }

            ThemeParkPlus.getEconomy().withdrawPlayer(e.getPlayer(), price);

            e.getPlayer().getInventory().addItem(ticket);
            e.getPlayer().updateInventory();
            e.getPlayer().sendMessage(ChatColor.GREEN + "Bought a Fastpass ticket for " + ConfigUtil.makecolored(att.getName()) + ChatColor.GREEN + ". You paid €" + String.format("%.2f", price) + " for it!");
        } else if (ChatColor.stripColor(sign.getLine(1)).equalsIgnoreCase("Scanner")) {
            //Scan a ticket
            if (!API.isAttraction(sign.getLine(2))) {
                e.getPlayer().sendMessage(ChatColor.RED + "This sign is incorrect! The attraction doesn't exists.");
                return;
            }
            Attraction att = API.getAttraction(sign.getLine(2));

            ItemStack content = e.getPlayer().getInventory().getItemInMainHand();
            NBTItem nbtContent = new NBTItem(content);
            if (content.getType() == Material.AIR
                || !content.hasItemMeta() || !nbtContent.hasKey("RideID")) {
                e.getPlayer().sendMessage(ChatColor.RED + "You need a fastpass ticket to go through the fastpass line.");
                return;
            }

            if (att.getStatus() != Status.OPEN && att.getStatus() != Status.ACTIVE) {
                e.getPlayer().sendMessage(ChatColor.RED + "This attraction is closed.");
                return;
            }

            try {
                String[] locationString = sign.getLine(3).split(":");
                int x = Integer.parseInt(locationString[0]);
                int y = Integer.parseInt(locationString[1]);
                int z = Integer.parseInt(locationString[2]);

                Block b = e.getPlayer().getWorld().getBlockAt(x, y, z);

                if (LGUtil.isOpenable(b)) {
                    LGUtil.openGate(b);
                    Bukkit.getScheduler().scheduleSyncDelayedTask(ThemeParkPlus.getInstance(), () -> LGUtil.closeGate(b), 60L);
                } else {
                    Location tpLoc = new Location(e.getPlayer().getWorld(), x, y, z, e.getPlayer().getLocation().getYaw(), e.getPlayer().getLocation().getPitch());
                    e.getPlayer().teleport(tpLoc);
                }
                e.getPlayer().getInventory().remove(e.getPlayer().getInventory().getItemInMainHand());
                e.getPlayer().updateInventory();
                e.getPlayer().sendMessage(ChatColor.GREEN + "Successfully redeemed your fastpass ticket!");
            } catch (NumberFormatException | ArrayIndexOutOfBoundsException ex) {
                e.getPlayer().sendMessage(ChatColor.RED + "This sign is incorrect!");
            }
        }
    }
}
